#memmap xmem
#class auto
// Usamos el hardware de lectura de la touch screen basado en ADS7846
#define TSCONTROLLER	2
/* Esto incluye la biblioteca de funciones de Cika que soporta el display
de 320x240 de Powertip */
#use "Cika320240FRST.lib"
/* Esta biblioteca provee funciones elementales que permiten utilizar
funciones ms complejas en las demostraciones sin modificar las
bibliotecas originales de Rabbit */
//#use "CikaDemoKeybuzz.lib"

#use FFT.lib

// Define to translate to 2's complement
//#define ACSCOPE

// analog samples
#define SAMPLES 256
// log2(SAMPLES/2)
#define L2_SAMPLES2	7

#define XSCALES 7
#define YSCALES 5
#define INPUT	0xE4

fontInfo fi8x10,fi10x12,fi6x8;

static unsigned int buffer[SAMPLES+2];		// sample buffer

unsigned  long userX;		// starting memory location where the user
									//	buttons will be stored.
int redraw;

void draw(float scale)
{
unsigned int i,y,lasty,aux;

  	glBuffLock();
	glSetBrushType(PIXWHITE);
	glBlock(1,8,256,224); 						// clear osc screen
	glSetBrushType(PIXBLACK);
  	aux=(unsigned int)(((long)scale*buffer[0])>>12);
   lasty=(aux<225)? 231-aux:8;				// "clamp to screen"
 	glPlotDot(1,y);
	for(i=1;i<SAMPLES;i++){
   	aux=(unsigned int)(((long)scale*buffer[i])>>12);
		y=(aux<225)? 231-aux:8;					// "clamp to screen"
  		glPlotLine(i,lasty,i+1,y);
      lasty=y;
   }
   glBuffUnlock();
}


cofunc sampledraw(int delta, float scale)
{
unsigned int k,y,lasty,aux;

	if(delta>=100){
		buffer[0]=_adcTouchScreen(INPUT);
  		aux=(unsigned int)(((long)scale*buffer[0])>>12);
	   lasty=(aux<225)? 231-aux:8;					// "clamp to screen";
 		glPlotDot(1,lasty);
		for(k=1;k<SAMPLES;k++){
   		waitfor(DelayMs(delta));
	     	if(redraw==2){
         	redraw=0;
   	     	abort;
         }
	  		glBuffLock();
			glSetBrushType(PIXWHITE);
			glBlock(k,8,(k<(SAMPLES-10))? 10:(SAMPLES-k),224);		// clear frame
			glSetBrushType(PIXBLACK);
			buffer[k]=_adcTouchScreen(INPUT);
   		aux=(unsigned int)(((long)scale*buffer[k])>>12);
			y=(aux<225)? 231-aux:8;					// "clamp to screen"
	  		glPlotLine(k,lasty,k+1,y);
		   glBuffUnlock();
      	lasty=y;
      }
   }
   else {
		for(k=0;k<SAMPLES;k++){
   		waitfor(DelayMs(delta));
	     	if(redraw==2){
         	redraw=0;
   	     	abort;
         }
			buffer[k]=_adcTouchScreen(INPUT);
	   }
		draw(scale);
   }
   return(1);
}


cofunc sampledrawfft(int delta, float scale)
{
int *bufferi,blockexp,i;
float power;

		for(i=0;i<SAMPLES;i++){
   		waitfor(DelayMs(delta));
	     	if(redraw==2){
         	redraw=0;
   	     	abort;
         }
			buffer[i]=_adcTouchScreen(INPUT);
	   }
      bufferi=(int *) buffer;			// translate to signed integer (fits)
		blockexp=0;
#ifdef ACSCOPE
      for(i=0;i<SAMPLES;i++)			// translate to 2's complement
        	bufferi[i]-=2048;
#endif

      fftreal(bufferi,SAMPLES/2,&blockexp);

    // realfft() returns the fmax term in the imaginary position of the dc
    // term, i.e. in x[1].  The values of the dc term in x[0] and fmax term in
    // x[1] equal the full amplitude of their respective components.  The
    // values of all the terms other than dc term and the fmax term equal one
    // half the amplitude of their respective components.  (The other half is
    // contained in the missing, negative-frequency components.)

		bufferi[0] /= 2;					// normalize DC term
		bufferi[1] = 0;					// discard fmax term

      blockexp-=L2_SAMPLES2;			// account for the 1/N
		blockexp*=2;						// squaring of all FFT components
      power=pow(2,blockexp);			// scaling term
	   for(i=0;i<SAMPLES/2;i++){		// get the FFT modules: ||=sqrt(Re^2+Im^2)
   	  	buffer[2*i]=(unsigned int) (sqrt((((float)bufferi[2*i]*bufferi[2*i])+((float)bufferi[2*i+1]*bufferi[2*i+1]))*power)+0.5);
         buffer[2*i+1]=buffer[2*i];
      }
     	draw(scale);

}


void main()
{
int btn,level,xscale,yscale;
const static float scale[]={224,560,1120,2240,5600};
const static int delta[]={1000,500,200,100,50,20,10};
const static char *scales[]={"5V fs   ","2V fs   ","1V fs   ","500mV fs","200mV fs"};
const static char *deltas[]={"1 sps  ","2 sps  ","5 sps  ","10 sps ","20 sps ","50 sps ","100 sps"};

   redraw=level=1;
   yscale=0;
   xscale=XSCALES-1;

	brdInit();
	glInit();

	glXFontInit ( &fi8x10,8,10,0x20,0x7E,Font8x10 );
	glXFontInit ( &fi10x12,10,12,0x20,0x7E,Terminal9 );
	glXFontInit ( &fi6x8,6,8,0x20,0x7E,Font6x8 );

	glBlankScreen();

	userX = btnInit( 7 );
	btnCreateText(userX,1,276,120,30,20,1,1,&fi8x10,"V+");
	btnCreateText(userX,2,276,141,30,20,1,1,&fi8x10,"V-");
	btnCreateText(userX,3,260,170,29,20,1,1,&fi8x10,"H-");
	btnCreateText(userX,4,290,170,29,20,1,1,&fi8x10,"H+");
	btnCreateText(userX,5,260,202,59,34,1,2,&fi10x12,"FFT");
	btnCreateText(userX,6,260,202,59,34,1,3,&fi10x12,"Osc");
	btnAttributes(userX,1,0,0,0,0);
	btnAttributes(userX,2,0,0,0,0);
	btnAttributes(userX,3,0,0,0,0);
	btnAttributes(userX,4,0,0,0,0);
	btnAttributes(userX,5,0,0,0,0);
	btnAttributes(userX,6,0,0,0,0);


   while (1) {
   	costate {		// control panel
     		if(redraw==1){
				glBlankScreen();
   	   	while (!btnDisplayLevel(userX,1)); // Will display all buttons of Level 1
      	   if(level==1){
					while (!btnClearLevel(userX,3)); 	// remove buttons of Level 3
            	while (!btnDisplayLevel(userX,2)); // Will display all buttons of Level 2
					glPrintf(280,0,&fi10x12,"OSC");
   	      }
      	   else {
					while (!btnClearLevel(userX,2)); 	// remove buttons of Level 2
	      		while (!btnDisplayLevel(userX,3)); // Will display all buttons of Level 3
					glPrintf(280,0,&fi10x12,"FFT");
   	      }
				glPlotPolygon(4,0,7,257,7,257,232,0,232); 		// draw osc screen
				glPrintf(270,20,&fi6x8,scales[yscale]);
				glPrintf(270,40,&fi6x8,deltas[xscale]);
	      	redraw=2;						// signal the scope it has to abort
   	   }
      }
		costate {	// botones
			waitfor ( (	btn = btnGet(userX) ) >= 0 );
			switch (btn){
				case 1:
            	if(++yscale==YSCALES)
               	yscale=YSCALES-1;
               redraw=2;
					break;
			 	case 2:
            	if(--yscale==-1)
               	yscale=0;
               redraw=2;
					break;
				case 3:
            	if(--xscale==-1)
               	xscale=0;
               redraw=2;
					break;
			 	case 4:
            	if(++xscale==XSCALES)
               	xscale=XSCALES-1;
               redraw=2;
					break;
			 	case 5:
            	level=2;
               redraw=1;
					break;
			 	case 6:
		         level=1;
               redraw=1;
               break;
			}
			glPrintf(270,20,&fi6x8,scales[yscale]);
			glPrintf(270,40,&fi6x8,deltas[xscale]);
		}
	    costate {     // scope
         if(level==1){
	         wfd sampledraw(delta[xscale],scale[yscale]);
         }
         else {
	         wfd sampledrawfft(delta[xscale],scale[yscale]);
       	}
   	}
   }
}

int buzzer()
{
}


